#include "xdmapcie.h"

XDmaPcie_Config XDmaPcie_ConfigTable[] __attribute__ ((section (".drvcfg_sec"))) = {

	{
		"amd,versal2-mdb-host", /* compatible */
		0xed931000, /* reg */
		0x2, /* xlnx,num-of-bars */
		0x0, /* xlnx,include-baroffset-reg */
		0x1, /* xlnx,port-type */
		0xed860000, /* xlnx,iatu-address */
		0x100000000000, /* xlnx,csr-slcr */
		0xa0000000,
		0xa7ffffff /* ranges */
	},
	 {
		 NULL
	}
};